unit TOTER_Edit;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, RefCtrls, NumEdits;

type
  TEditDialog = class(TForm)
    OKButton: TButton;
    CancelButton: TButton;
    LocationLbl: TLabel;
    LocationEdt: TRefDirectoryEdit;
    BitbucketBtn: TButton;
    IncludeLbl: TLabel;
    IncludeEdt: TEdit;
    ExcludeLbl: TLabel;
    ExcludeEdt: TEdit;
    LapsedDaysLbl: TLabel;
    BaseDateCbx: TComboBox;
    LapsedTimeEdt: TUpDownEdit;
    AttributeGroup: TGroupBox;
    ReadOnlyChk: TCheckBox;
    HiddenChk: TCheckBox;
    SysFileChk: TCheckBox;
    DirectoryChk: TCheckBox;
    ArchiveChk: TCheckBox;
    OptionGroup: TGroupBox;
    SubdirTooChk: TCheckBox;
    EmptydirOnlyChk: TCheckBox;
    DeadlnkNowChk: TCheckBox;
    IgnoreUninsChk: TCheckBox;
    ToBitbucketChk: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure LocationEdtChange(Sender: TObject);
    procedure BitbucketBtnClick(Sender: TObject);
    procedure FileEdtsChange(Sender: TObject);
  private
    { Private 錾 }
    procedure UpdateOKButton;
  public
    { Public 錾 }
  end;

var
  EditDialog: TEditDialog;

implementation

{$R *.dfm}

uses
  TOTER_Main,
  ShellUtils, Winapi.ShlObj;

procedure TEditDialog.UpdateOKButton;
begin
  OKButton.Enabled := (LocationEdt.Text <> '') and (IncludeEdt.Text <> '')
    and (LapsedTimeEdt.Relation = nrAccord);
end;

procedure TEditDialog.FormShow(Sender: TObject);
var
  s: string;
  i: Integer;
begin
  with MainForm.PatternView.ItemFocused do
  begin
    LocationEdt.Text := Caption;
    IncludeEdt.Text := SubItems.Strings[0];
    ExcludeEdt.Text := SubItems.Strings[1];

    s := SubItems.Strings[2];
    if s = '' then
    begin
      BaseDateCbx.ItemIndex := 2;
      LapsedTimeEdt.Value := 7;
    end
    else
    begin
      case s[1] of
        'C': BaseDateCbx.ItemIndex := 0;
        'M': BaseDateCbx.ItemIndex := 1;
        'A': BaseDateCbx.ItemIndex := 2;
        'D': BaseDateCbx.ItemIndex := 3;
      end;
      LapsedTimeEdt.Value := StrToInt(Copy(s, 3, Length(s)));
    end;
    for i := 0 to AttributeGroup.ControlCount - 1 do
      TCheckBox(AttributeGroup.Controls[i]).Checked := False;

    s := SubItems.Strings[3];
    for i := 1 to Length(s) do
      case s[i] of
        'R': ReadOnlyChk.Checked := True;
        'H': HiddenChk.Checked := True;
        'S': SysFileChk.Checked := True;
        'D': DirectoryChk.Checked := True;
        'A': ArchiveChk.Checked := True;
      end;

    for i := 0 to OptionGroup.ControlCount - 1 do
      TCheckBox(OptionGroup.Controls[i]).Checked := False;
    s := SubItems.Strings[4];
    for i := 1 to Length(s) do
      case s[i] of
        'T': SubdirTooChk.Checked := True;
        'O': EmptydirOnlyChk.Checked := True;
        'I': DeadlnkNowChk.Checked := True;
        'U': IgnoreUninsChk.Checked := True;
        'B': ToBitbucketChk.Checked := True;
      end;

    UpdateOKButton;
  end;
end;

procedure TEditDialog.FormClose(Sender: TObject; var Action: TCloseAction);
var
  s: string;
begin
  if ModalResult = mrOK then
  begin
    with MainForm.PatternView.ItemFocused do
    begin
      Caption := LocationEdt.Text;
      SubItems.Strings[0] := IncludeEdt.Text;
      SubItems.Strings[1] := ExcludeEdt.Text;

      case BaseDateCbx.ItemIndex of
        0: s := 'C+';
        1: s := 'M+';
        2: s := 'A+';
        3: s := 'D+';
      end;
      SubItems.Strings[2] := s + IntToStr(LapsedTimeEdt.Value);

      s := '';
      if ReadOnlyChk.Checked then
        s := s + 'R';
      if HiddenChk.Checked then
        s := s + 'H';
      if SysFileChk.Checked then
        s := s + 'S';
      if DirectoryChk.Checked then
        s := s + 'D';
      if ArchiveChk.Checked then
        s := s + 'A';
      SubItems.Strings[3] := s;

      s := '';
      if SubdirTooChk.Checked then
        s := s + 'T';
      if EmptydirOnlyChk.Checked and DirectoryChk.Checked then
        s := s + 'O';
      if DeadlnkNowChk.Checked then
        s := s + 'I';
      if IgnoreUninsChk.Checked and DeadlnkNowChk.Checked then
        s := s + 'U';
      if ToBitbucketChk.Checked then
        s := s + 'B';
      SubItems.Strings[4] := s;
    end;
  end;
end;

procedure TEditDialog.LocationEdtChange(Sender: TObject);
var
  Selectable: Boolean;
  i: Integer;
begin
  if LocationEdt.Text = GetSpecialFolderPath(CSIDL_BITBUCKET) then
    Selectable := False
  else
    Selectable := True;
  if Selectable then
  begin
    if BaseDateCbx.Items.Count = 4 then
      BaseDateCbx.Items.Delete(3);
    BaseDateCbx.ItemIndex := 2;
    BaseDateCbx.Enabled := True;
  end
  else
  begin
    if BaseDateCbx.Items.Count = 3 then
      BaseDateCbx.Items.Add('폜');
    BaseDateCbx.ItemIndex := 3;
    BaseDateCbx.Enabled := False;
  end;

  for i := 0 to AttributeGroup.ControlCount - 1 do
    TCheckBox(AttributeGroup.Controls[i]).Enabled := Selectable;
  for i := 0 to OptionGroup.ControlCount - 1 do
    TCheckBox(OptionGroup.Controls[i]).Enabled := Selectable;

  UpdateOKButton;
end;

procedure TEditDialog.BitbucketBtnClick(Sender: TObject);
begin
  LocationEdt.Text := GetSpecialFolderPath(CSIDL_BITBUCKET);
end;

procedure TEditDialog.FileEdtsChange(Sender: TObject);
begin
  UpdateOKButton
end;

end.
