unit TOTER_Main;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.ImageList, Vcl.ImgList, Vcl.Menus, Vcl.ToolWin,
  Vcl.ComCtrls, Vcl.ExtCtrls, Vcl.StdCtrls, Winapi.ShlObj, TOTER_Remove;

type
  TMainForm = class(TForm)
    OpenDlg: TOpenDialog;
    SaveDlg: TSaveDialog;
    MenuImages: TImageList;
    MainMenu: TMainMenu;
    FileMnu: TMenuItem;
    FileNewMnu: TMenuItem;
    FileOpenMnu: TMenuItem;
    FileSaveMnu: TMenuItem;
    FileSaveAsMnu: TMenuItem;
    FileN1Mnu: TMenuItem;
    FileExitMnu: TMenuItem;
    ItemMnu: TMenuItem;
    ItemAddMnu: TMenuItem;
    ItemDeleteMnu: TMenuItem;
    ItemModifyMnu: TMenuItem;
    ItemN1Mnu: TMenuItem;
    ItemUpMnu: TMenuItem;
    ItemDownMnu: TMenuItem;
    DeleteMnu: TMenuItem;
    DeleteTestMnu: TMenuItem;
    DeleteRunMnu: TMenuItem;
    DeleteN1Mnu: TMenuItem;
    DeleteStopMnu: TMenuItem;
    ViewMnu: TMenuItem;
    ViewClearMnu: TMenuItem;
    OptionMnu: TMenuItem;
    OptionConfigMnu: TMenuItem;
    ToolBar: TToolBar;
    FileNewBtn: TToolButton;
    FileOpenBtn: TToolButton;
    FileSaveBtn: TToolButton;
    ToolN1Btn: TToolButton;
    ItemAddBtn: TToolButton;
    ItemDeleteBtn: TToolButton;
    ItemModifyBtn: TToolButton;
    ToolN3Btn: TToolButton;
    ItemUpBtn: TToolButton;
    ItemDownBtn: TToolButton;
    ToolN2Btn: TToolButton;
    DeleteTestBtn: TToolButton;
    DeleteRunBtn: TToolButton;
    ToolN4Btn: TToolButton;
    DeleteStopBtn: TToolButton;
    PatternView: TListView;
    Splitter: TSplitter;
    LogView: TMemo;
    StatusBar: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure FileNewMnuClick(Sender: TObject);
    procedure FileOpenMnuClick(Sender: TObject);
    procedure FileSaveMnuClick(Sender: TObject);
    procedure FileSaveAsMnuClick(Sender: TObject);
    procedure FileExitMnuClick(Sender: TObject);
    procedure ItemAddMnuClick(Sender: TObject);
    procedure ItemDeleteMnuClick(Sender: TObject);
    procedure ItemModifyMnuClick(Sender: TObject);
    procedure ItemUpMnuClick(Sender: TObject);
    procedure ItemDownMnuClick(Sender: TObject);
    procedure DeleteTestMnuClick(Sender: TObject);
    procedure DeleteRunMnuClick(Sender: TObject);
    procedure DeleteStopMnuClick(Sender: TObject);
    procedure ViewClearMnuClick(Sender: TObject);
    procedure OptionConfigMnuClick(Sender: TObject);
    procedure PatternViewInsert(Sender: TObject; Item: TListItem);
    procedure PatternViewChange(Sender: TObject; Item: TListItem; Change: TItemChange);
    procedure PatternViewContextPopup(Sender: TObject; MousePos: TPoint; var Handled: Boolean);
    procedure PatternViewDblClick(Sender: TObject);
    procedure PatternViewKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure PatternViewDeletion(Sender: TObject; Item: TListItem);
  private
    { Private 錾 }
    FRemoveThread: TRemoveThread;
    FRun: Boolean;
    FPatternFile: string;
    FModified: Boolean;
    FProgress: ITaskbarList3; //Vcl.Taskbar.TTaskbar ͍ŏŋN ETaskbarException B
    procedure SetPatternFile(Value: string);
    procedure SetModified(Value: Boolean);
    procedure ReadOption;
    procedure WriteOption;
    procedure UpdateItems;
    procedure InitFile;
    procedure LoadFile(FileName: string);
    procedure SaveFile(FileName: string);
    function SaveQuery: Integer;
    procedure SaveLogFile;
  public
    { Public 錾 }
    SaveLog: Boolean;
    LogFile: string;
    LogLinage: Integer;
    procedure RemoveThreadExecute(TestRun: Boolean);
    procedure RemoveThreadTerminate(Sender: TObject);
    procedure ApplicationIdleAtOnce(Sender: TObject; var Done: Boolean);
    property PatternFile: string read FPatternFile write SetPatternFile;
    property Modified: Boolean read FModified write SetModified;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
  TOTER_Edit, TOTER_Config,
  System.UITypes, AppData, Winapi.ActiveX;

resourcestring
  SUntitled = '';
  SModified = 'ύX';
  SSelection = '%d/%d I';
  SSaveConfirmation = '%s ̕ύXۑ܂H';
  SRemoveConfirmation = 't@C폜s܂H';

procedure TMainForm.SetPatternFile(Value: string);
const
  AppName = 'Totter';
begin
  if Value <> FPatternFile then
  begin
    FPatternFile := Value;
    if FPatternFile = '' then
      Caption := Format('%s - %s', [SUntitled, AppName])
    else
      Caption := Format('%s - %s', [ExtractFileName(FPatternFile), AppName]);
  end;
end;

procedure TMainForm.SetModified(Value: Boolean);
begin
  if Value = FModified then Exit;

  FModified := Value;
  if FModified then
    StatusBar.Panels.Items[1].Text := SModified
  else
    StatusBar.Panels.Items[1].Text := '';
end;

procedure TMainForm.ReadOption;
var
  IniFile: TAppIniFile;
begin
  IniFile := TAppIniFile.Create(AppDataName('ShaggyDog\Totter\Totter.ini'));
  try
    SaveLog := IniFile.ReadBool('Log', 'Save', False);
    LogFile := IniFile.ReadString('Log', 'File', AppDataName('ShaggyDog\Totter\Totter.log'));
    LogLinage := IniFile.ReadInteger('Log', 'Linage', 256);
  finally
    IniFile.Free;
  end;
end;

procedure TMainForm.WriteOption;
var
  IniFile: TAppIniFile;
begin
  IniFile := TAppIniFile.Create(AppDataName('ShaggyDog\Totter\Totter.ini'));
  try
    IniFile.WriteBool('Log', 'Save', SaveLog);
    IniFile.WriteString('Log', 'File', LogFile);
    IniFile.WriteInteger('Log', 'Linage', LogLinage);
  finally
    IniFile.Free;
  end;
end;

procedure TMainForm.UpdateItems;
var
  Selectable: Boolean;
begin
  Selectable := Assigned(PatternView.ItemFocused) and not Assigned(FRemoveThread);
  ItemDeleteMnu.Enabled := Selectable;
  ItemModifyMnu.Enabled := Selectable;
  ItemUpMnu.Enabled := Selectable;
  ItemDownMnu.Enabled := Selectable;
  ItemDeleteBtn.Enabled := Selectable;
  ItemModifyBtn.Enabled := Selectable;
  ItemUpBtn.Enabled := Selectable;
  ItemDownBtn.Enabled := Selectable;

  Selectable := PatternView.SelCount > 0;
  DeleteTestMnu.Enabled := Selectable;
  DeleteRunMnu.Enabled := Selectable;
  DeleteTestBtn.Enabled := Selectable;
  DeleteRunBtn.Enabled := Selectable;

  Selectable := Assigned(FRemoveThread);
  DeleteStopMnu.Enabled := Selectable;
  DeleteStopBtn.Enabled := Selectable;

  with PatternView do
    StatusBar.Panels.Items[0].Text := Format(SSelection, [SelCount, Items.Count]);
end;

procedure TMainForm.InitFile;
begin
  PatternFile := '';
  PatternView.Items.Clear;
  UpdateItems;
  Modified := False;
end;

procedure TMainForm.LoadFile(FileName: string);
var
  IniFile: TAppIniFile;
  Sections: TStrings;
  i: Integer;
  Seting: string;
  NewItem: TListItem;
begin
  IniFile := TAppIniFile.Create(FileName);
  try
    PatternView.Items.Clear;
    Sections := TStringList.Create;
    try
      IniFile.ReadSections(Sections);
      for i := 0 to Sections.Count - 1 do
      begin
        Seting := Sections.Strings[i];
        NewItem := PatternView.Items.Add;
        with NewItem do
        begin
          Caption := IniFile.ReadString(Seting, 'Location', '');
          SubItems.Add(IniFile.ReadString(Seting, 'Mask', ''));
          SubItems.Add(IniFile.ReadString(Seting, 'Except', ''));
          SubItems.Add(IniFile.ReadString(Seting, 'LapsedDays', ''));
          SubItems.Add(IniFile.ReadString(Seting, 'Attribute', ''));
          SubItems.Add(IniFile.ReadString(Seting, 'Option', ''));
        end;
      end;
      UpdateItems;
      PatternFile := FileName;
      Modified := False;
    finally
      Sections.Free;
    end;
  finally
    IniFile.Free;
  end;
end;

procedure TMainForm.SaveFile(FileName: string);
var
  IniFile: TAppIniFile;
  i: Integer;
  Seting: string;
begin
  if FileName = '' then
    if SaveDlg.Execute then
      FileName := SaveDlg.FileName
    else
      Exit;

  IniFile := TAppIniFile.Create(FileName);
  try
    IniFile.Clear;
    for i := 0 to PatternView.Items.Count - 1 do
    begin
      Seting := 'Schedule' + IntToStr(i);
      with PatternView.Items.Item[i] do
      begin
        IniFile.WriteString(Seting, 'Location', Caption);
        IniFile.WriteString(Seting, 'Mask', SubItems.Strings[0]);
        IniFile.WriteString(Seting, 'Except', SubItems.Strings[1]);
        IniFile.WriteString(Seting, 'LapsedDays', SubItems.Strings[2]);
        IniFile.WriteString(Seting, 'Attribute', SubItems.Strings[3]);
        IniFile.WriteString(Seting, 'Option', SubItems.Strings[4]);
      end;
    end;
    PatternFile := FileName;
    Modified := False;
  finally
    IniFile.Free;
  end;
end;

function TMainForm.SaveQuery: Integer;
var
  FileName: string;
begin
  Result := mrNone;
  if Modified then
  begin
    FileName := ExtractFileName(FPatternFile);
    if FileName = '' then
      FileName := SUntitled;
    Result := MessageDlg(Format(SSaveConfirmation, [FileName]), mtInformation, mbYesNoCancel, 0);
  end;
end;

procedure TMainForm.SaveLogFile;
var
  Report: TStrings;
begin
  Report := TStringList.Create;
  try
    if FileExists(LogFile) then
      Report.LoadFromFile(LogFile);
    Report.AddStrings(LogView.Lines);
    Report.BeginUpdate;
    try
      while Report.Count > LogLinage do
        Report.Delete(0);
    finally
      Report.EndUpdate;
    end;
    Report.SaveToFile(LogFile);
  finally
    Report.Free;
  end;
end;

procedure TMainForm.RemoveThreadExecute(TestRun: Boolean);
begin
  FRemoveThread := TRemoveThread.Create(PatternView, FProgress, LogView, TestRun);
  FRemoveThread.OnTerminate := RemoveThreadTerminate;

  UpdateItems;
end;

procedure TMainForm.RemoveThreadTerminate(Sender: TObject);
begin
  FRemoveThread := nil;
  if FRun then
    Close
  else
    UpdateItems;
end;

procedure TMainForm.ApplicationIdleAtOnce(Sender: TObject; var Done: Boolean);
begin
  if Assigned(Application.OnIdle) then
  begin
    Application.OnIdle := nil;

    if PatternView.Items.Count > 0 then
    begin
      PatternView.SelectAll;

      RemoveThreadExecute(False);
    end;
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  ReadOption;
  FRun := False;
  FPatternFile := '';
  FModified := False;
  FProgress := nil;
  if TOSVersion.Check(6, 1) then
    CoCreateInstance(CLSID_TaskbarList, nil, CLSCTX_ALL, IID_ITaskbarList3, FProgress);

  if ParamCount = 0 then
    InitFile
  else
  begin
    for i := 1 to ParamCount do
    begin
      if CharInSet(ParamStr(i)[1], ['-', '/']) then
      begin
        case ParamStr(i)[2] of
          'r', 'R': FRun := True;
          'w', 'W':
          begin
            if Pos('+', ParamStr(i)) = 3 then
              WindowState := wsMaximized
            else
            if Pos('-', ParamStr(i)) = 3 then
            begin
              WindowState := wsMinimized;
              Application.Minimize;
            end;
          end;
        end;
      end
      else
        LoadFile(ParamStr(i));
    end;
    if FRun then
      Application.OnIdle := ApplicationIdleAtOnce;
  end;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  DeleteStopMnu.Click;

  case SaveQuery of
    mrCancel: CanClose := False;
    mrYes: SaveFile(FPatternFile);
  end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  if SaveLog then
    SaveLogFile;
end;

procedure TMainForm.FileNewMnuClick(Sender: TObject);
begin
  DeleteStopMnu.Click;

  case SaveQuery of
    mrCancel: Exit;
    mrYes: SaveFile(FPatternFile);
  end;
  InitFile;
end;

procedure TMainForm.FileOpenMnuClick(Sender: TObject);
begin
  DeleteStopMnu.Click;

  case SaveQuery of
    mrCancel: Exit;
    mrYes: SaveFile(FPatternFile);
  end;
  if OpenDlg.Execute then
    LoadFile(OpenDlg.FileName);
end;

procedure TMainForm.FileSaveMnuClick(Sender: TObject);
begin
  SaveFile(PatternFile);
end;

procedure TMainForm.FileSaveAsMnuClick(Sender: TObject);
begin
  SaveDlg.FileName := PatternFile;
  if SaveDlg.Execute then
    SaveFile(SaveDlg.FileName);
end;

procedure TMainForm.FileExitMnuClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.ItemAddMnuClick(Sender: TObject);
var
  NewItem: TListItem;
  i: Integer;
begin
  NewItem := PatternView.Items.Add;
  for i := 0 to 4 do
    NewItem.SubItems.Add('');
  PatternView.ItemFocused := NewItem;
  if EditDialog.ShowModal = mrOk then
    Modified := True
  else
    NewItem.Delete;
  UpdateItems;
end;

procedure TMainForm.ItemDeleteMnuClick(Sender: TObject);
begin
  PatternView.ItemFocused.Delete;
  UpdateItems;
  Modified := True;
end;

procedure TMainForm.ItemModifyMnuClick(Sender: TObject);
begin
  if EditDialog.ShowModal = mrOk then
    Modified := True;
end;

procedure TMainForm.ItemUpMnuClick(Sender: TObject);
var
  Index: Integer;
  Item: TListItem;
begin
  Index := PatternView.ItemFocused.Index;
  if Index < 1 then
    Index := PatternView.Items.Count
  else
    Dec(Index);
  Item := PatternView.Items.Insert(Index);
  Item.Assign(PatternView.ItemFocused);
  PatternView.ItemFocused.Delete;
  PatternView.ItemFocused := Item;
  Modified := True;
end;

procedure TMainForm.ItemDownMnuClick(Sender: TObject);
var
  Index: Integer;
  Item: TListItem;
begin
  Index := PatternView.ItemFocused.Index;
  if Index >= PatternView.Items.Count - 1 then
    Index := 0
  else
    Inc(Index, 2);
  Item := PatternView.Items.Insert(Index);
  Item.Assign(PatternView.ItemFocused);
  PatternView.ItemFocused.Delete;
  PatternView.ItemFocused := Item;
  Modified := True;
end;

procedure TMainForm.DeleteTestMnuClick(Sender: TObject);
begin
  DeleteStopMnu.Click;

  RemoveThreadExecute(True);
end;

procedure TMainForm.DeleteRunMnuClick(Sender: TObject);
begin
  DeleteStopMnu.Click;

  if MessageDlg(SRemoveConfirmation, mtConfirmation, mbYesNo, 0) = mrYes then
    RemoveThreadExecute(False);
end;

procedure TMainForm.DeleteStopMnuClick(Sender: TObject);
begin
  if Assigned(FRemoveThread) then
    FRemoveThread.Terminate;
end;

procedure TMainForm.ViewClearMnuClick(Sender: TObject);
begin
  if SaveLog then
    SaveLogFile;
  LogView.Lines.Clear;
end;

procedure TMainForm.OptionConfigMnuClick(Sender: TObject);
begin
  if ConfigDialog.ShowModal = mrOk then
    WriteOption;
end;

procedure TMainForm.PatternViewInsert(Sender: TObject; Item: TListItem);
begin
  if Assigned(Item) then
    Item.Data := TStringList.Create;
end;

procedure TMainForm.PatternViewDeletion(Sender: TObject; Item: TListItem);
begin
  if Assigned(Item) and Assigned(Item.Data) then
    TStringList(Item.Data).Free;
end;

procedure TMainForm.PatternViewChange(Sender: TObject; Item: TListItem; Change: TItemChange);
begin
  UpdateItems;
end;

procedure TMainForm.PatternViewContextPopup(Sender: TObject; MousePos: TPoint;
  var Handled: Boolean);
begin
  TrackPopupMenuEx(ItemMnu.Handle, TPM_RECURSE, Mouse.CursorPos.X, Mouse.CursorPos.Y, Handle, nil);
end;

procedure TMainForm.PatternViewDblClick(Sender: TObject);
begin
  ItemModifyMnu.Click;
end;

procedure TMainForm.PatternViewKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Key = Word('A')) and (ssCtrl in Shift) then
    PatternView.SelectAll;
end;

end.
