unit TOTER_Remove;

interface

uses
  System.Classes, Winapi.Windows, Vcl.ComCtrls, Winapi.ShlObj, Vcl.StdCtrls,
  System.Generics.Collections, Winapi.ShellAPI;

type
  TOptionSet = set of AnsiChar;

  TRemoveThread = class(TThread)
  private
    { Private 錾 }
    FPatterns: TListView;
    FProgress: ITaskbarList3;
    FReport: TMemo;
    FTestRun: Boolean;
    FFlags: Integer;
    FTotal,
    FCompleted: UInt64;
    FListItem: TListItem;
    FLocation,
    FIncludes,
    FExcludes: string;
    FBaseDate: Char;
    FLapsedTime: TDateTime;
    FAttrib: Integer;
    FOption: TOptionSet;
    FReservation: TStrings;
    FDeletion: TStrings;
    FResultMsg: string;
    procedure BeginProgress;
    procedure StepProgress;
    procedure EndProgress;
    procedure NextSchedule;
    procedure InitSchedule;
    procedure CompleteSchedule;
    procedure PopulateReservation;
    procedure PopulateDeletion;
    procedure ScanBitbucket;
    procedure ScanFile;
    function FileDeleteOperation(FileName: string; Flags: FILEOP_FLAGS): Integer;
    function FormatReport(Stamp: TDateTime; Command: string; Code: Integer; FileName: string): string;
    procedure RemoveBitbucket;
    procedure RemoveFile;
  protected
    { protected 錾 }
    procedure Execute; override;
  public
    { public 錾 }
    constructor Create(Patterns: TListView; Progress: ITaskbarList3; Report: TMemo;
      TestRun: Boolean); virtual;
    destructor Destroy; override;
  end;

implementation

{ 
  dv: rWA R|[lgɂIuWFNg̃\bhƃvpeB́ASynchronize g
  Ăяo郁\bhł̂ݎgpł܂BƂ΁Â悤ɂȂ܂B

      Synchronize(UpdateCaption);  

  UpdateCaption ́AƂΎ̂悤ȃR[hɂȂ܂B

    procedure TRemoveThread.UpdateCaption;
    begin
      Form1.Caption := 'XbhōXV܂';
    end; 
    
    邢 
    
    Synchronize( 
      procedure 
      begin
        Form1.Caption := '\bhʂăXbhōXV܂' 
      end
      )
    );
    
  ł́A\bhnĂ܂B
  
  lɁAJ҂͏LƓ悤ȃp[^ Queue \bhĂяoƂł܂B
  Aʂ TThread NX 1 p[^ƂēnAĂяõXbh
  ̃XbhŃL[ɓ܂B
    
}

uses
  System.SysUtils, System.Win.ComObj, ShellTypes, ShellUtils, FileUtils, System.IOUtils, ShellLink;

const
  OperationFlags = FOF_SILENT or FOF_NOCONFIRMATION or FOF_NOERRORUI;

{ TRemoveThread }

procedure TRemoveThread.BeginProgress;
begin
  if Assigned(FProgress) then
    FProgress.SetProgressState(FPatterns.Parent.Handle, FFlags);

  FListItem := FPatterns.GetNextItem(nil, sdAll, [isSelected]);
end;

procedure TRemoveThread.StepProgress;
begin
  Inc(FCompleted);
  if Assigned(FProgress) then
    FProgress.SetProgressValue(FPatterns.Parent.Handle, FCompleted, FTotal);

  FReport.Lines.Add(FResultMsg);
end;

procedure TRemoveThread.EndProgress;
begin
  if Assigned(FProgress) then
    FProgress.SetProgressState(FPatterns.Parent.Handle, FFlags);
end;

procedure TRemoveThread.NextSchedule;
begin
  FListItem := FPatterns.GetNextItem(FListItem, sdAll, [isSelected]);
end;

procedure TRemoveThread.InitSchedule;
var
  s: string;
  i: Integer;
begin
  if Assigned(FListItem) and FListItem.Selected then
  begin
    with FListItem do
    begin
      FLocation := Caption;
      FIncludes := SubItems.Strings[0];
      FExcludes := SubItems.Strings[1];

      s := SubItems.Strings[2];
      FBaseDate := s[1];
      FLapsedTime := Now - StrToInt(Copy(s, 3, Length(s)));

      FAttrib := 0;
      s := SubItems.Strings[3];
      for i := 1 to Length(s) do
        case s[i] of
          'R': Inc(FAttrib, faReadOnly);
          'H': Inc(FAttrib, faHidden);
          'S': Inc(FAttrib, faSysFile);
          'D': Inc(FAttrib, faDirectory);
          'A': Inc(FAttrib, faArchive);
        end;

      FOption := [];
      s := SubItems.Strings[4];
      for i := 1 to Length(s) do
        Include(FOption, AnsiChar(s[i]));

      FReservation := TStringList(Data);
    end;
  end
  else
  begin
    FLocation := '';
    FIncludes := '';
    FExcludes := '';
    FBaseDate := #0;
    FLapsedTime := 0;
    FAttrib := 0;
    FOption := [];
    FReservation := nil;
  end;
end;

procedure TRemoveThread.CompleteSchedule;
begin
  if Assigned(FListItem) then
    FListItem.Selected := False;
end;

procedure TRemoveThread.PopulateReservation;
begin
  if Assigned(FReservation) then
    FReservation.Assign(FDeletion);
end;

procedure TRemoveThread.PopulateDeletion;
begin
  if Assigned(FReservation) then
    FDeletion.Assign(FReservation);
end;

procedure TRemoveThread.ScanBitbucket;
const
  EnumFlags = SHCONTF_FOLDERS or SHCONTF_NONFOLDERS or SHCONTF_INCLUDEHIDDEN;
var
  BitbucketID, FileID: PItemIDList;
  BitbucketFolder: IShellFolder2;
  EnumList: IEnumIDList;
  NumIDs: LongWord;
  FilePath, FileName: string;
  Column: SHCOLUMNID;
  UTCTime: OleVariant;
  TimeStamp: TDateTime;
begin
  SHGetSpecialFolderLocation(0, CSIDL_BITBUCKET, BitbucketID);
  try
    OleCheck(DesktopShellFolder.BindToObject(BitbucketID, nil, IID_IShellFolder2, BitbucketFolder));
    OleCheck(BitbucketFolder.EnumObjects(0, EnumFlags, EnumList));
    while not Terminated and (EnumList.Next(1, FileID, NumIDs) = S_OK) do
      try
        FilePath := GetParseName(BitbucketFolder, FileID);
        FileName := GetFolderName(BitbucketFolder, FileID);
        if MatchesCompoundMasks(FileName, FIncludes, FExcludes) then
        begin
          Column.fmtid := FMTID_Displaced;
          Column.pid := PID_DISPLACED_DATE;
          if BitbucketFolder.GetDetailsEx(FileID, Column, @UTCTime) = S_OK then
            TimeStamp := StrToLocalDateTime(UTCTime)
          else
            TimeStamp := Now;
          if TimeStamp <= FLapsedTime then
            FDeletion.Add(FilePath + '=' + FileName);
        end;
      finally
        DisposePIDL(FileID);
      end;
  finally
    DisposePIDL(BitbucketID);
  end;
end;

procedure TRemoveThread.ScanFile;
var
  FindAttr, FindResult: Integer;
  Exclusion, FileName, LinkedFile: string;
  SearchRec: TSearchRec;
  TimeStamp: TDateTime;
begin
  if DirectoryExists(FLocation) then
  begin
    FindAttr := FAttrib;
    if CharInSet('T', FOption) and ((FindAttr and faDirectory) = 0) then
      Inc(FindAttr, faDirectory);

    Exclusion := '.;..';
    if FExcludes <> '' then
      Exclusion := Exclusion + ';' + FExcludes;

    FindResult := FindFirst(TPath.Combine(FLocation, '*'), FindAttr, SearchRec);
    try
      while not Terminated and (FindResult = 0) do
      begin
        if MatchesCompoundMasks(SearchRec.Name, FIncludes, Exclusion) then
        begin
          FileName := TPath.Combine(FLocation, SearchRec.Name);
          case FBaseDate of
            'C': TimeStamp := FileTimeToLocalDateTime(SearchRec.FindData.ftCreationTime);
            'M': TimeStamp := FileTimeToLocalDateTime(SearchRec.FindData.ftLastWriteTime);
            'A': TimeStamp := FileTimeToLocalDateTime(SearchRec.FindData.ftLastAccessTime);
          else
            TimeStamp := FLapsedTime;
          end;

          if CharInSet('O', FOption) and HasDirectory(SearchRec) then
          begin
            if not TDirectory.IsEmpty(FileName) then
              TimeStamp := Now;
          end;

          if TPath.MatchesPattern(SearchRec.Name, '*.lnk', False) then
          begin
            LinkedFile := GetLinkTarget(FileName);
            if not (DirectoryExists(LinkedFile) or FileExists(LinkedFile))
              and CharInSet('I', FOption) then
            begin
              case GetDriveType(PChar(ExtractFileDrive(LinkedFile) + '\')) of
                DRIVE_REMOVABLE, DRIVE_CDROM:
                begin
                  if not CharInSet('U', FOption) then
                    TimeStamp := FLapsedTime;
                end;
              else
                TimeStamp := FLapsedTime;
              end;
            end;
          end;

          if TimeStamp <= FLapsedTime then
            FDeletion.Add(FileName);
        end;
        FindResult := FindNext(SearchRec);
      end;
    finally
      FindClose(SearchRec);
    end;
  end;
end;

function TRemoveThread.FileDeleteOperation(FileName: string; Flags: FILEOP_FLAGS): Integer;
var
  FileOpStruct: TSHFileOpStruct;
begin
  ZeroMemory(@FileOpStruct, SizeOf(TSHFileOpStruct));
  with FileOpStruct do
  begin
    wnd := Handle;
    wFunc := FO_DELETE;
    pFrom := PChar(FileName + #0#0);
    pTo := '';
    fFlags := Flags;
    fAnyOperationsAborted := False;
    hNameMappings := nil;
    lpszProgressTitle := nil;
  end;
  Result := SHFileOperation(FileOpStruct);
end;

function TRemoveThread.FormatReport(Stamp: TDateTime; Command: string; Code: Integer;
  FileName: string): string;
const
  Form = 'yyyy/mm/dd hh:nn:ss';
begin
  //Result := Format('%s %s %5d %s', [FormatDateTime(Form, Stamp), Command, Code, FileName]);
  Result := Format('%s %s 0x%.4x %s', [FormatDateTime(Form, Stamp), Command, Code, FileName]);
end;

procedure TRemoveThread.RemoveBitbucket;
var
  i: Integer;
  FilePath, FileName, Command: string;
  Code: Integer;
begin
  for i := 0 to FDeletion.Count - 1 do
  begin
    FilePath := FDeletion.Names[i];
    FileName := TPath.Combine(ExtractFilePath(FilePath), FDeletion.ValueFromIndex[i]);
    if FTestRun then
    begin
      Command := 'TE';
      Code := 0;
    end
    else
    begin
      Command := 'RM';
      Code := FileDeleteOperation(FilePath, OperationFlags);
    end;
    FResultMsg := FormatReport(Now, Command, Code, FileName);
    Synchronize(StepProgress);
  end;
  //XP ̏ꍇASHFileOperation ō폜ƂɂݔɂȂĂACRςȂB
  //dȂ̂ NULL t@ĈĂăACRXVB
  if not FTestRun then
    FileDeleteOperation('', OperationFlags or FOF_ALLOWUNDO);  //o[W`FbN͏ȗB
end;

procedure TRemoveThread.RemoveFile;
var
  i: Integer;
  FileName: string;
  FileOpFlags: FILEOP_FLAGS;
  Command: string;
  Code: Integer;
begin
  for i := 0 to FDeletion.Count - 1 do
  begin
    FileName := FDeletion.Strings[i];
    if FTestRun then
    begin
      Command := 'TE';
      Code := 0;
    end
    else
    begin
      Command := 'RM';
      FileOpFlags := OperationFlags;
      if CharInSet('B', FOption) then
        FileOpFlags := FileOpFlags or FOF_ALLOWUNDO;
      Code := FileDeleteOperation(FileName, FileOpFlags);
    end;
    FResultMsg := FormatReport(Now, Command, Code, FileName);
    Synchronize(StepProgress);
  end;
end;

procedure TRemoveThread.Execute;
begin
  NameThreadForDebugging('RemoveThd');
  { XbhƂĎsR[hɋLqĂ }
  FFlags := TBPF_INDETERMINATE;
  Synchronize(BeginProgress);
  while not Terminated and Assigned(FListItem) do
  begin
    Synchronize(InitSchedule);
    if FLocation <> '' then
    begin
      FDeletion.Clear;
      if FLocation = GetSpecialFolderPath(CSIDL_BITBUCKET) then
        ScanBitbucket
      else
        ScanFile;
      Inc(FTotal, FDeletion.Count);
      Synchronize(PopulateReservation);
    end;
    Synchronize(NextSchedule);
  end;
  if not Terminated then
  begin
    FFlags := TBPF_NORMAL;
    Synchronize(BeginProgress);
  end;
  while not Terminated and Assigned(FListItem) do
  begin
    Synchronize(InitSchedule);
    if FLocation <> '' then
    begin
      Synchronize(PopulateDeletion);
      if FLocation = GetSpecialFolderPath(CSIDL_BITBUCKET) then
        RemoveBitbucket
      else
        RemoveFile;
      Synchronize(CompleteSchedule);
    end;
    Synchronize(NextSchedule);
  end;
  FFlags := TBPF_NOPROGRESS;
  Synchronize(EndProgress);
end;

constructor TRemoveThread.Create(Patterns: TListView; Progress: ITaskbarList3; Report: TMemo;
  TestRun: Boolean);
begin
  FPatterns := Patterns;
  FProgress := Progress;
  FReport := Report;
  FTestRun := TestRun;
  FFlags := TBPF_NOPROGRESS;
  FTotal := 0;
  FCompleted := 0;
  FListItem := nil;
  InitSchedule;
  FDeletion := TStringList.Create;

  inherited Create(False);
  FreeOnTerminate := True;
end;

destructor TRemoveThread.Destroy;
begin
  FDeletion.Free;

  inherited Destroy;
end;

end.
